/** \file
	\brief FFT window: rectangular (no window)
*/

#include "FFT/FFTWindow.h"
#include "FFT/FFTWindowFactory.h"
#include "FFT/FFTWindowDescription.h"

namespace
{
	FFTWindowDescription desc(
		"Rectangular",
		"Rectangular window (or no window) is preferred for broadband random "
		"(white noise), transient or synchrounously samples signals. Best frequency resolution, worst spectrum leakage.",
		0.88f);

	class FFTWindowRectangular: public FFTWindow
	{
	private:
		friend FFTWindow* CreateWindow(void);
		FFTWindowRectangular(const FFTWindowDescription &desc):
			FFTWindow(desc)
		{};
	public:
		virtual void Resize(unsigned int size);
		virtual void Process(float *data);
	};

	FFTWindow* CreateWindow(void)
	{
		return new FFTWindowRectangular(desc);
	}

	const bool registered =
		FFTWindowFactory::Instance().Register(desc, CreateWindow);
}

void FFTWindowRectangular::Resize(unsigned int size)
{

}

void FFTWindowRectangular::Process(float *data)
{

}


