//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Ms3ViewerUnit.h"
#include "Ms3FrameListUnit.h"
#include "DataFrame.h"
#include "display\ValueStringFormatter.h"
#include "display\Display.h"
#include "device\MiniscopeDeviceInterface.h"
#include "Ms3Recorder.h"
#include "Log.h"

#include <fstream>
#include <time.h>
#include <stdio.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMs3Viewer *frmMs3Viewer;
//---------------------------------------------------------------------------
__fastcall TfrmMs3Viewer::TfrmMs3Viewer(TComponent* Owner)
	: TForm(Owner),
	callbackClose(NULL),
	data(NULL),
	frame_cnt(0)
{
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::FormClose(TObject *Sender, TCloseAction &Action)
{
	frmMs3FrameList->Close();
	if (callbackClose)
	{
		callbackClose();
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::btnExitViewerClick(TObject *Sender)
{
	Close();	
}
//---------------------------------------------------------------------------

int TfrmMs3Viewer::LoadFile(AnsiString asFileName, CallbackUpdate cbUpdate, float *fDivX,
	std::vector<float> *vdata)
{
	data = vdata;
	int status = Ms3Recorder::Instance().SetMode(Ms3Recorder::MODE_READ, asFileName);
	if (status)
		return status;
	asCurrentFile = asFileName;
	edFilename->Text = asCurrentFile;
	this->fDivX = fDivX;
	this->callbackUpdate = cbUpdate;
	frame_cnt = Ms3Recorder::Instance().GetFrameCount();
	data->clear();
	if (frame_cnt > 0)
	{
		frmMs3FrameList->selection.first = 0;
		frmMs3FrameList->selection.count = 1;
		frmMs3FrameList->UpdateView();
		status = LoadFrames(frmMs3FrameList->selection.first, frmMs3FrameList->selection.count);
	}
	else
	{
		frmMs3FrameList->selection.first = 0;
		frmMs3FrameList->selection.count = 0;
		frmMs3FrameList->UpdateView();
		callbackUpdate();
		lblFrameVal->Caption = "---";
		status = LoadFrames(frmMs3FrameList->selection.first, frmMs3FrameList->selection.count);
	}

	return status;
}

int TfrmMs3Viewer::LoadFrames(int first, int count)
{
	if (count == 0)
	{
		AnsiString asFrameVal;
		asFrameVal.sprintf("--- of %d", frame_cnt);
		lblFrameVal->Caption = asFrameVal;
		lblSamplingVal->Caption = "---";
		lblTriggerTypeVal->Caption = "---";
		lblCouplingTypeVal->Caption = "---";
		lblTimestampVal->Caption = "---";
		lblSamplesCountVal->Caption = "---";
		callbackUpdate();
		return 0;
	}

	struct S_DATAFRAME desc;
	std::vector<float> tmp;
	int status;
	data->clear();
	for (int i=0; i<count; i++)
	{
		status = Ms3Recorder::Instance().GetFrame(first+i, desc, &tmp);
		if (status)
		{
			LOG(E_LOG_TRACE, "Failed to get data frame (status = %d)", status);
			return status;
		}
		else
		{
			if (i == 0)
			{
				float fSamplVal = (float)1.0 / desc.fDivX;
				lblSamplingVal->Caption =
					ValueStringFormatter::FreqToStr(fSamplVal, 1).c_str();

				lblTriggerTypeVal->Caption = DeviceInterface::GetTriggerTypeName((E_TRIGGER_TYPE)desc.trigger_type);
				lblCouplingTypeVal->Caption = DeviceInterface::GetCouplingTypeName((E_COUPLING_TYPE)desc.coupling);
				char* timeline = ctime( & ( desc.timestamp.time ) );
				timeline[19]=0;
				char buf[64];
				sprintf(buf, "%15s.%03hu", &timeline[4], desc.timestamp.millitm);
				lblTimestampVal->Caption = buf;
				lblSamplesCountVal->Caption = desc.buffer_size;
			}

			data->reserve(data->size() + tmp.size());
			for (unsigned int i=0; i<tmp.size(); i++)
			{
				data->push_back(tmp[i]);
			}
		}
	}

	*fDivX = desc.fDivX;
	callbackUpdate();

	AnsiString asFrameVal;
	if (count > 1)
		asFrameVal.sprintf("%d...%d of %d", first+1, first+count, frame_cnt);
	else
		asFrameVal.sprintf("%d of %d", first+1, frame_cnt);

	lblFrameVal->Caption = asFrameVal;

	return status;
}

/** Behaviour: one frame added, one removed from selection (if data looks continuous)
*/
void __fastcall TfrmMs3Viewer::btnPrevFrameClick(TObject *Sender)
{
	if (frame_cnt == 0 || frmMs3FrameList->selection.first <= 0 || frmMs3FrameList->selection.count == 0)
		return;
	int id = frmMs3FrameList->selection.first - 1;
	struct S_DATAFRAME desc;
	int status = Ms3Recorder::Instance().GetFrameDesc(id, desc);
	if (status)
	{
		LOG(E_LOG_TRACE, "Ms3Viewer: failed to get desc for frame %d", id);
		frmMs3FrameList->selection.first = 0;
		frmMs3FrameList->selection.count = 0;
		LoadFrames(frmMs3FrameList->selection.first, frmMs3FrameList->selection.count);
		frmMs3FrameList->UpdateView();
		return;
	}
	/** \todo Create function common with TfrmMs3FrameList */
	// check frame parameters consistency
	// currently only same sampling frequency is required
	if (desc.fDivX != frmMs3FrameList->selection.desc.fDivX)
	{
		frmMs3FrameList->selection.first = id;
		frmMs3FrameList->selection.count = 1;
	}
	else
	{
		frmMs3FrameList->selection.first = id;
	}

	LoadFrames(frmMs3FrameList->selection.first, frmMs3FrameList->selection.count);
	frmMs3FrameList->UpdateView();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::btnNextFrameClick(TObject *Sender)
{
	if (frame_cnt == 0 ||
		(frmMs3FrameList->selection.first + frmMs3FrameList->selection.count >= (int)frame_cnt) ||
		frmMs3FrameList->selection.count == 0)
		return;
	int id = frmMs3FrameList->selection.first + frmMs3FrameList->selection.count;
	struct S_DATAFRAME desc;
	int status = Ms3Recorder::Instance().GetFrameDesc(id, desc);
	if (status)
	{
		LOG(E_LOG_TRACE, "Ms3Viewer: failed to get desc for frame %d", id);
		frmMs3FrameList->selection.first = 0;
		frmMs3FrameList->selection.count = 0;
		LoadFrames(frmMs3FrameList->selection.first, frmMs3FrameList->selection.count);
		frmMs3FrameList->UpdateView();
		return;
	}
	/** \todo Create function common with TfrmMs3FrameList */
	// check frame parameters consistency
	// currently only same sampling frequency is required
	if (desc.fDivX != frmMs3FrameList->selection.desc.fDivX)
	{
		frmMs3FrameList->selection.first = id;
		frmMs3FrameList->selection.count = 1;
	}
	else
	{
		frmMs3FrameList->selection.first++;
	}

	LoadFrames(frmMs3FrameList->selection.first, frmMs3FrameList->selection.count);
	frmMs3FrameList->UpdateView();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::btnFrameListClick(TObject *Sender)
{
	frmMs3FrameList->Show();	
}
//---------------------------------------------------------------------------

